"use strict";
/*
* This program and the accompanying materials are made available under the terms of the *
* Eclipse Public License v2.0 which accompanies this distribution, and is available at *
* https://www.eclipse.org/legal/epl-v20.html                                      *
*                                                                                 *
* SPDX-License-Identifier: EPL-2.0                                                *
*                                                                                 *
* Copyright Contributors to the Zowe Project.                                     *
*                                                                                 *
*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const imperative_1 = require("@zowe/imperative");
const nls = require("vscode-nls");
// Localization support
const localize = nls.config({ messageFormat: nls.MessageFormat.file })(__filename);
/**
 * Keytar - Securely store user credentials in the system keychain
 *
 * @export
 * @class KeytarCredentialManager
 */
class KeytarCredentialManager extends imperative_1.AbstractCredentialManager {
    /**
     * Pass-through to the superclass constructor.
     *
     * @param {string} service The service string to send to the superclass constructor.
     * @param {string} displayName The display name for this credential manager to send to the superclass constructor
     */
    constructor(service, displayName) {
        // Always ensure that a manager instantiates the super class, even if the
        // constructor doesn't do anything. Who knows what things might happen in
        // the abstract class initialization in the future.
        super(service, displayName);
        /**
         * Combined list of services that credentials may be stored under
         */
        this.allServices = ["Zowe-Plugin", "@brightside/core", "@zowe/cli", "Broadcom-Plugin"];
        if (this.allServices.indexOf(service) === -1) {
            this.allServices.push(service);
        }
        this.preferredService = service;
    }
    /**
     * Calls the keytar deletePassword service with {@link DefaultCredentialManager#service} and the
     * account passed to the function by Imperative.
     *
     * @param {string} account The account for which to delete the password
     *
     * @returns {Promise<void>} A promise that the function has completed.
     *
     * @throws {@link ImperativeError} if keytar is not defined.
     * @throws {@link ImperativeError} when keytar.deletePassword returns false.
     */
    deleteCredentials(account) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!(yield this.deleteCredentialsHelper(account))) {
                throw new imperative_1.ImperativeError({
                    msg: localize(0, null),
                    additionalDetails: this.getMissingEntryMessage(account)
                });
            }
        });
    }
    /**
     * Calls the keytar getPassword service with {@link DefaultCredentialManager#service} and the
     * account passed to the function by Imperative.
     *
     * @param {string} account The account for which to get credentials
     * @returns {Promise<SecureCredential>} A promise containing the credentials stored in keytar.
     *
     * @throws {@link ImperativeError} if keytar is not defined.
     * @throws {@link ImperativeError} when keytar.getPassword returns null or undefined.
     */
    loadCredentials(account) {
        return __awaiter(this, void 0, void 0, function* () {
            // Helper function to handle all breaking changes
            const loadHelper = (service) => __awaiter(this, void 0, void 0, function* () {
                let secureValue = yield KeytarCredentialManager.keytar.getPassword(service, account);
                // Handle user vs username case // Zowe v1 -> v2 (i.e. @brightside/core@2.x -> @zowe/cli@6+ )
                if (secureValue == null && account.endsWith("_username")) {
                    secureValue = yield KeytarCredentialManager.keytar.getPassword(service, account.replace("_username", "_user"));
                }
                // Handle pass vs password case // Zowe v0 -> v1 (i.e. @brightside/core@1.x -> @brightside/core@2.x)
                if (secureValue == null && account.endsWith("_pass")) {
                    secureValue = yield KeytarCredentialManager.keytar.getPassword(service, account.replace("_pass", "_password"));
                }
                return secureValue;
            });
            let password;
            // Check for stored credentials under each of the known services
            // We will stop checking once we find them somewhere
            for (const service of this.allServices) {
                password = yield loadHelper(service);
                if (password != null) {
                    break;
                }
            }
            // Throw an error if credentials could not be found
            if (password == null) {
                throw new imperative_1.ImperativeError({
                    msg: localize(1, null),
                    additionalDetails: this.getMissingEntryMessage(account)
                });
            }
            return password;
        });
    }
    /**
     * Calls the keytar setPassword service with {@link DefaultCredentialManager#service} and the
     * account and credentials passed to the function by Imperative.
     *
     * @param {string} account The account to set credentials
     * @param {SecureCredential} credentials The credentials to store
     *
     * @returns {Promise<void>} A promise that the function has completed.
     *
     * @throws {@link ImperativeError} if keytar is not defined.
     */
    saveCredentials(account, credentials) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.deleteCredentialsHelper(account, true);
            yield KeytarCredentialManager.keytar.setPassword(this.preferredService, account, credentials);
        });
    }
    deleteCredentialsHelper(account, skipPreferredService) {
        return __awaiter(this, void 0, void 0, function* () {
            let wasDeleted = false;
            for (const service of this.allServices) {
                if (skipPreferredService && service === this.preferredService) {
                    continue;
                }
                if (yield KeytarCredentialManager.keytar.deletePassword(service, account)) {
                    wasDeleted = true;
                }
            }
            return wasDeleted;
        });
    }
    getMissingEntryMessage(account) {
        return localize(2, null) +
            localize(3, null, this.allServices.join(", ")) +
            localize(4, null, account) +
            localize(5, null) +
            localize(6, null) +
            localize(7, null) +
            localize(8, null);
    }
}
exports.KeytarCredentialManager = KeytarCredentialManager;

//# sourceMappingURL=KeytarCredentialManager.js.map
