"use strict";
/*
* This program and the accompanying materials are made available under the terms of the *
* Eclipse Public License v2.0 which accompanies this distribution, and is available at *
* https://www.eclipse.org/legal/epl-v20.html                                      *
*                                                                                 *
* SPDX-License-Identifier: EPL-2.0                                                *
*                                                                                 *
* Copyright Contributors to the Zowe Project.                                     *
*                                                                                 *
*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
/**
 * Standard history and favorite persistance handling routines
 *
 * @export
 * @class PersistentFilters
 */
class PersistentFilters {
    constructor(schema, maxHistory = 5) {
        this.maxHistory = maxHistory;
        this.mHistory = [];
        this.mSessions = [];
        this.schema = schema;
        this.initialize();
    }
    /**
     * Retrieves a generic setting either in user or workspace.
     * <pre>{@code
     *  PersistentFilters.getDirectValue("Zowe Commands: Always edit") as boolean;
     * }</pre>
     * @param key - string. The attribute value that needs retrieving
     */
    static getDirectValue(key) {
        const settings = Object.assign({}, vscode.workspace.getConfiguration());
        return settings.get(key);
    }
    readFavorites() {
        if (vscode.workspace.getConfiguration(this.schema)) {
            return vscode.workspace.getConfiguration(this.schema).get(PersistentFilters.favorites);
        }
        return [];
    }
    updateFavorites(favorites) {
        return __awaiter(this, void 0, void 0, function* () {
            // settings are read-only, so were cloned
            const settings = Object.assign({}, vscode.workspace.getConfiguration(this.schema));
            if (settings.persistence) {
                settings.favorites = favorites;
                yield vscode.workspace.getConfiguration().update(this.schema, settings, vscode.ConfigurationTarget.Global);
            }
        });
    }
    /**
     * Adds one line of history to the local store and
     * updates persistent store. The store contains a
     * maximum number of entries as described by `maxHistory`
     *
     * If the entry matches a previous entry it is removed from the list
     * at that position in the stack.
     *
     * Once the maximum capacity has been reached the last entry is popped off
     *
     * @param {string} criteria - a line of search criteria
     */
    addHistory(criteria) {
        return __awaiter(this, void 0, void 0, function* () {
            if (criteria) {
                // Remove any entries that match
                this.mHistory = this.mHistory.filter((element) => {
                    return element.trim() !== criteria.trim();
                });
                // Add value to front of stack
                this.mHistory.unshift(criteria);
                // If list getting too large remove last entry
                if (this.mHistory.length > this.maxHistory) {
                    this.mHistory.pop();
                }
                this.updateHistory();
            }
        });
    }
    getHistory() {
        return this.mHistory;
    }
    resetHistory() {
        return __awaiter(this, void 0, void 0, function* () {
            this.mHistory = [];
            this.updateHistory();
        });
    }
    /**
     * Adds one line of history to the local store and
     * updates persistent store.
     *
     * If the entry matches a previous entry it is removed from the list
     * at that position in the stack.
     *
     * @param {string} criteria - a session name
     */
    addSession(criteria) {
        return __awaiter(this, void 0, void 0, function* () {
            // Remove any entries that match
            this.mSessions = this.mSessions.filter((element) => {
                return element.trim() !== criteria.trim();
            });
            this.mSessions.push(criteria);
            // Use standard sorting
            this.mSessions.sort();
            this.updateSessions();
        });
    }
    removeSession(criteria) {
        return __awaiter(this, void 0, void 0, function* () {
            // Remove any entries that match
            this.mSessions = this.mSessions.filter((element) => {
                return element.trim() !== criteria.trim();
            });
            this.updateSessions();
        });
    }
    getSessions() {
        return this.mSessions;
    }
    resetSessions() {
        return __awaiter(this, void 0, void 0, function* () {
            this.mSessions = [];
            this.updateSessions();
        });
    }
    /**
     * Initializes the history and sessions sections by reading from a file
     */
    initialize() {
        return __awaiter(this, void 0, void 0, function* () {
            let lines;
            if (vscode.workspace.getConfiguration(this.schema)) {
                lines = vscode.workspace.getConfiguration(this.schema).get(PersistentFilters.history);
            }
            if (lines) {
                this.mHistory = lines;
            }
            else {
                this.resetHistory();
            }
            if (vscode.workspace.getConfiguration(this.schema)) {
                lines = vscode.workspace.getConfiguration(this.schema).get(PersistentFilters.sessions);
            }
            if (lines) {
                this.mSessions = lines;
            }
            else {
                this.resetSessions();
            }
        });
    }
    updateHistory() {
        return __awaiter(this, void 0, void 0, function* () {
            // settings are read-only, so make a clone
            const settings = Object.assign({}, vscode.workspace.getConfiguration(this.schema));
            if (settings.persistence) {
                settings.history = this.mHistory;
                yield vscode.workspace.getConfiguration().update(this.schema, settings, vscode.ConfigurationTarget.Global);
            }
        });
    }
    updateSessions() {
        return __awaiter(this, void 0, void 0, function* () {
            // settings are read-only, so make a clone
            const settings = Object.assign({}, vscode.workspace.getConfiguration(this.schema));
            if (settings.persistence) {
                settings.sessions = this.mSessions;
                yield vscode.workspace.getConfiguration().update(this.schema, settings, vscode.ConfigurationTarget.Global);
            }
        });
    }
}
exports.PersistentFilters = PersistentFilters;
PersistentFilters.favorites = "favorites";
PersistentFilters.history = "history";
PersistentFilters.sessions = "sessions";

//# sourceMappingURL=PersistentFilters.js.map
