"use strict";
/*
* This program and the accompanying materials are made available under the terms of the *
* Eclipse Public License v2.0 which accompanies this distribution, and is available at *
* https://www.eclipse.org/legal/epl-v20.html                                      *
*                                                                                 *
* SPDX-License-Identifier: EPL-2.0                                                *
*                                                                                 *
* Copyright Contributors to the Zowe Project.                                     *
*                                                                                 *
*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const extension = require("./extension");
const ZoweTreeNode_1 = require("./abstract/ZoweTreeNode");
const utils = require("./utils");
const ZoweExplorerApiRegister_1 = require("./api/ZoweExplorerApiRegister");
const nls = require("vscode-nls");
const icons_1 = require("./generators/icons");
const localize = nls.config({ messageFormat: nls.MessageFormat.file })(__filename);
// tslint:disable-next-line: max-classes-per-file
class Job extends ZoweTreeNode_1.ZoweTreeNode {
    constructor(label, collapsibleState, mParent, session, job, profile) {
        super(label, collapsibleState, mParent, session, profile);
        this.job = job;
        this.children = [];
        this.dirty = true;
        if (session) {
            this._owner = session.ISession.user;
        }
        this._prefix = "*";
        this._searchId = "";
        const icon = icons_1.getIconByNode(this);
        if (icon) {
            this.iconPath = icon.path;
        }
    }
    /**
     * Retrieves child nodes of this IZoweJobTreeNode
     *
     * @returns {Promise<IZoweJobTreeNode[]>}
     */
    getChildren() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.dirty) {
                let spools = [];
                const elementChildren = [];
                if (this.contextValue === extension.JOBS_JOB_CONTEXT || this.contextValue === extension.JOBS_JOB_CONTEXT + extension.FAV_SUFFIX) {
                    spools = yield vscode.window.withProgress({
                        location: vscode.ProgressLocation.Notification,
                        title: localize(0, null)
                    }, () => {
                        return ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getJesApi(this.getProfile()).getSpoolFiles(this.job.jobname, this.job.jobid);
                    });
                    spools.forEach((spool) => {
                        const existing = this.children.find((element) => element.label.trim() === `${spool.stepname}:${spool.ddname}(${spool.id})`);
                        if (existing) {
                            elementChildren.push(existing);
                        }
                        else {
                            let prefix = spool.stepname;
                            if (prefix === undefined) {
                                prefix = spool.procstep;
                            }
                            const sessionName = this.contextValue === extension.JOBS_JOB_CONTEXT + extension.FAV_SUFFIX ?
                                this.label.substring(1, this.label.lastIndexOf("]")).trim() :
                                this.getProfileName();
                            const spoolNode = new Spool(`${spool.stepname}:${spool.ddname}(${spool.id})`, vscode.TreeItemCollapsibleState.None, this, this.session, spool, this.job, this);
                            const icon = icons_1.getIconByNode(spoolNode);
                            if (icon) {
                                spoolNode.iconPath = icon.path;
                            }
                            spoolNode.command = { command: "zowe.zosJobsOpenspool", title: "", arguments: [sessionName, spool] };
                            elementChildren.push(spoolNode);
                        }
                    });
                }
                else {
                    const jobs = yield vscode.window.withProgress({
                        location: vscode.ProgressLocation.Notification,
                        title: localize(1, null)
                    }, () => {
                        return this.getJobs(this._owner, this._prefix, this._searchId);
                    });
                    jobs.forEach((job) => {
                        let nodeTitle;
                        if (job.retcode) {
                            nodeTitle = `${job.jobname}(${job.jobid}) - ${job.retcode}`;
                        }
                        else {
                            nodeTitle = `${job.jobname}(${job.jobid}) - ${job.status}`;
                        }
                        const existing = this.children.find((element) => element.label.trim() === nodeTitle);
                        if (existing) {
                            elementChildren.push(existing);
                        }
                        else {
                            const jobNode = new Job(nodeTitle, vscode.TreeItemCollapsibleState.Collapsed, this, this.session, job, this.getProfile());
                            jobNode.command = { command: "zowe.zosJobsSelectjob", title: "", arguments: [jobNode] };
                            jobNode.contextValue = extension.JOBS_JOB_CONTEXT;
                            if (!jobNode.iconPath) {
                                const icon = icons_1.getIconByNode(jobNode);
                                if (icon) {
                                    jobNode.iconPath = icon.path;
                                }
                            }
                            elementChildren.push(jobNode);
                        }
                    });
                }
                elementChildren.sort((a, b) => {
                    if (a.job.jobid > b.job.jobid) {
                        return 1;
                    }
                    if (a.job.jobid < b.job.jobid) {
                        return -1;
                    }
                    return 0;
                });
                this.children = elementChildren;
            }
            this.dirty = false;
            return this.children;
        });
    }
    getSessionNode() {
        return this.getParent() ? this.getParent().getSessionNode() : this;
    }
    get tooltip() {
        if (this.job !== null) {
            if (this.job.retcode) {
                return `${this.job.jobname}(${this.job.jobid}) - ${this.job.retcode}`;
            }
            else {
                return `${this.job.jobname}(${this.job.jobid})`;
            }
        }
        else if (this.searchId.length > 0) {
            return `${this.label} - job id: ${this.searchId}`;
        }
        else {
            return `${this.label} - owner: ${this.owner} prefix: ${this.prefix}`;
        }
    }
    set owner(newOwner) {
        if (newOwner !== undefined) {
            if (newOwner.length === 0) {
                this._owner = this.session.ISession.user;
            }
            else {
                this._owner = newOwner;
            }
        }
    }
    get owner() {
        return this._owner;
    }
    set prefix(newPrefix) {
        if (newPrefix !== undefined) {
            if (newPrefix.length === 0) {
                this._prefix = "*";
            }
            else {
                this._prefix = newPrefix;
            }
        }
    }
    get prefix() {
        return this._prefix;
    }
    set searchId(newId) {
        if (newId !== undefined) {
            this._searchId = newId;
        }
    }
    get searchId() {
        return this._searchId;
    }
    getJobs(owner, prefix, searchId) {
        return __awaiter(this, void 0, void 0, function* () {
            let jobsInternal = [];
            if (this.searchId.length > 0) {
                jobsInternal.push(yield ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getJesApi(this.getProfile()).getJob(searchId));
            }
            else {
                try {
                    jobsInternal = yield ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getJesApi(this.getProfile()).getJobsByOwnerAndPrefix(owner, prefix);
                }
                catch (error) {
                    yield utils.errorHandling(error, this.label, localize(2, null) + `zowe.GetJobs`);
                }
            }
            return jobsInternal;
        });
    }
}
exports.Job = Job;
Job.JobId = "JobId:";
Job.Owner = "Owner:";
Job.Prefix = "Prefix:";
// tslint:disable-next-line: max-classes-per-file
class Spool extends Job {
    constructor(label, mCollapsibleState, mParent, session, spool, job, parent) {
        super(label, mCollapsibleState, mParent, session, job, parent.getProfile());
        this.contextValue = extension.JOBS_SPOOL_CONTEXT;
        const icon = icons_1.getIconByNode(this);
        if (icon) {
            this.iconPath = icon.path;
        }
    }
}

//# sourceMappingURL=ZoweJobNode.js.map
