"use strict";
/*
* This program and the accompanying materials are made available under the terms of the *
* Eclipse Public License v2.0 which accompanies this distribution, and is available at *
* https://www.eclipse.org/legal/epl-v20.html                                      *
*                                                                                 *
* SPDX-License-Identifier: EPL-2.0                                                *
*                                                                                 *
* Copyright Contributors to the Zowe Project.                                     *
*                                                                                 *
*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const zowe = require("@zowe/cli");
const vscode = require("vscode");
const nls = require("vscode-nls");
const extension = require("../extension");
const Profiles_1 = require("../Profiles");
const PersistentFilters_1 = require("../PersistentFilters");
const utils_1 = require("../utils");
const localize = nls.config({ messageFormat: nls.MessageFormat.file })(__filename);
/**
 * Provides a class that manages submitting a command on the server
 *
 * @export
 * @class MvsCommandHandler
 */
class MvsCommandHandler {
    constructor() {
        this.outputChannel = vscode.window.createOutputChannel(localize(0, null));
        this.history = new PersistentFilters_1.PersistentFilters(MvsCommandHandler.persistenceSchema, MvsCommandHandler.totalFilters);
    }
    /**
     * Implements access singleton
     * for {MvsCommandHandler}.
     *
     * @returns {MvsCommandHandler}
     */
    static getInstance() {
        if (!MvsCommandHandler.instance) {
            MvsCommandHandler.instance = new MvsCommandHandler();
        }
        return this.instance;
    }
    /**
     * Allow the user to submit a MVS Console command to the selected server. Response is written
     * to the output channel.
     * @param session the session the command is to run against (optional) user is prompted if not supplied
     * @param command the command string (optional) user is prompted if not supplied
     */
    issueMvsCommand(session, command) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!session) {
                let usrNme;
                let passWrd;
                let baseEncd;
                const profiles = Profiles_1.Profiles.getInstance();
                const allProfiles = profiles.allProfiles;
                const profileNamesList = allProfiles.map((temprofile) => {
                    return temprofile.name;
                });
                if (profileNamesList.length) {
                    const quickPickOptions = {
                        placeHolder: localize(1, null),
                        ignoreFocusOut: true,
                        canPickMany: false
                    };
                    const sesName = yield vscode.window.showQuickPick(profileNamesList, quickPickOptions);
                    const zosmfProfile = allProfiles.filter((temprofile) => temprofile.name === sesName)[0];
                    const updProfile = zosmfProfile.profile;
                    if ((!updProfile.user) || (!updProfile.password)) {
                        try {
                            const values = yield Profiles_1.Profiles.getInstance().promptCredentials(zosmfProfile.name);
                            if (values !== undefined) {
                                usrNme = values[0];
                                passWrd = values[1];
                                baseEncd = values[2];
                            }
                        }
                        catch (error) {
                            vscode.window.showErrorMessage(error.message);
                        }
                        if (usrNme !== undefined && passWrd !== undefined && baseEncd !== undefined) {
                            updProfile.user = usrNme;
                            updProfile.password = passWrd;
                            updProfile.base64EncodedAuth = baseEncd;
                        }
                    }
                    session = zowe.ZosmfSession.createBasicZosmfSession(updProfile);
                }
                else {
                    vscode.window.showInformationMessage(localize(2, null));
                    return;
                }
            }
            let command1 = command;
            if (!command) {
                command1 = yield this.getQuickPick(session && session.ISession ? session.ISession.hostname : "unknown");
            }
            yield this.issueCommand(session, command1);
        });
    }
    getQuickPick(hostname) {
        return __awaiter(this, void 0, void 0, function* () {
            let response = "";
            const alwaysEdit = PersistentFilters_1.PersistentFilters.getDirectValue("Zowe Commands: Always edit");
            if (this.history.getHistory().length > 0) {
                const createPick = new utils_1.FilterDescriptor(MvsCommandHandler.defaultDialogText);
                const items = this.history.getHistory().map((element) => new utils_1.FilterItem(element));
                if (extension.ISTHEIA) {
                    const options1 = {
                        placeHolder: localize(3, null) + hostname +
                            (alwaysEdit ? localize(4, null) : "")
                    };
                    // get user selection
                    const choice = (yield vscode.window.showQuickPick([createPick, ...items], options1));
                    if (!choice) {
                        vscode.window.showInformationMessage(localize(5, null));
                        return;
                    }
                    response = choice === createPick ? "" : choice.label;
                }
                else {
                    const quickpick = vscode.window.createQuickPick();
                    quickpick.placeholder = alwaysEdit ?
                        localize(6, null) + hostname +
                            localize(7, null) :
                        localize(8, null) + hostname;
                    quickpick.items = [createPick, ...items];
                    quickpick.ignoreFocusOut = true;
                    quickpick.show();
                    const choice = yield utils_1.resolveQuickPickHelper(quickpick);
                    quickpick.hide();
                    if (!choice) {
                        vscode.window.showInformationMessage(localize(9, null));
                        return;
                    }
                    if (choice instanceof utils_1.FilterDescriptor) {
                        if (quickpick.value) {
                            response = quickpick.value;
                        }
                    }
                    else {
                        response = choice.label;
                    }
                }
            }
            if (!response || alwaysEdit) {
                // manually entering a search
                const options2 = {
                    prompt: localize(10, null),
                    value: response,
                    valueSelection: response ? [response.length, response.length] : undefined
                };
                // get user input
                response = yield vscode.window.showInputBox(options2);
                if (!response) {
                    vscode.window.showInformationMessage(localize(11, null));
                    return;
                }
            }
            return response;
        });
    }
    /**
     * Allow the user to submit an MVS Console command to the selected server. Response is written
     * to the output channel.
     * @param session The Session object
     * @param command the command string
     */
    issueCommand(session, command) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (command) {
                    // If the user has started their command with a / then remove it
                    if (command.startsWith("/")) {
                        command = command.substring(1);
                    }
                    this.outputChannel.appendLine(`> ${command}`);
                    const submitResponse = yield vscode.window.withProgress({
                        location: vscode.ProgressLocation.Notification,
                        title: localize(12, null)
                    }, () => {
                        return zowe.IssueCommand.issueSimple(session, command);
                    });
                    if (submitResponse.success) {
                        this.outputChannel.appendLine(submitResponse.commandResponse);
                        this.outputChannel.show(true);
                    }
                }
            }
            catch (error) {
                yield utils_1.errorHandling(error, null, error.message);
            }
            this.history.addHistory(command);
        });
    }
}
exports.MvsCommandHandler = MvsCommandHandler;
MvsCommandHandler.totalFilters = 10;
MvsCommandHandler.persistenceSchema = "Zowe Commands: History";
MvsCommandHandler.defaultDialogText = "\uFF0B " + localize(13, null);

//# sourceMappingURL=MvsCommandHandler.js.map
