"use strict";
/*
* This program and the accompanying materials are made available under the terms of the *
* Eclipse Public License v2.0 which accompanies this distribution, and is available at *
* https://www.eclipse.org/legal/epl-v20.html                                      *
*                                                                                 *
* SPDX-License-Identifier: EPL-2.0                                                *
*                                                                                 *
* Copyright Contributors to the Zowe Project.                                     *
*                                                                                 *
*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const zowe = require("@zowe/cli");
const fs = require("fs");
const utils = require("../utils");
const nls = require("vscode-nls");
const localize = nls.config({ messageFormat: nls.MessageFormat.file })(__filename);
const extension = require("../../src/extension");
const path = require("path");
const Profiles_1 = require("../Profiles");
const ZoweExplorerApiRegister_1 = require("../api/ZoweExplorerApiRegister");
const isbinaryfile_1 = require("isbinaryfile");
/**
 * Prompts the user for a path, and populates the [TreeView]{@link vscode.TreeView} based on the path
 *
 * @param {ZoweUSSNode} node - The session node
 * @param {ussTree} ussFileProvider - Current ussTree used to populate the TreeView
 * @returns {Promise<void>}
 */
function createUSSNode(node, ussFileProvider, nodeType, isTopLevel) {
    return __awaiter(this, void 0, void 0, function* () {
        const name = yield vscode.window.showInputBox({
            placeHolder: localize(0, null)
        });
        if (name) {
            try {
                const filePath = `${node.fullPath}/${name}`;
                yield ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getUssApi(node.getProfile()).create(filePath, nodeType);
                if (isTopLevel) {
                    refreshAllUSS(ussFileProvider);
                }
                else {
                    ussFileProvider.refreshElement(node);
                }
            }
            catch (err) {
                utils.errorHandling(err, node.mProfileName, localize(1, null) + err.message);
                throw (err);
            }
            ussFileProvider.refresh();
        }
    });
}
exports.createUSSNode = createUSSNode;
function createUSSNodeDialog(node, ussFileProvider) {
    return __awaiter(this, void 0, void 0, function* () {
        let usrNme;
        let passWrd;
        let baseEncd;
        let validProfile = -1;
        if ((!node.getSession().ISession.user.trim()) || (!node.getSession().ISession.password.trim())) {
            try {
                const values = yield Profiles_1.Profiles.getInstance().promptCredentials(node.mProfileName);
                if (values !== undefined) {
                    usrNme = values[0];
                    passWrd = values[1];
                    baseEncd = values[2];
                }
            }
            catch (error) {
                utils.errorHandling(error, node.mProfileName, localize(2, null) + `createUSSNodeDialog.optionalProfiles!`);
                return;
            }
            if (usrNme !== undefined && passWrd !== undefined && baseEncd !== undefined) {
                node.getSession().ISession.user = usrNme;
                node.getSession().ISession.password = passWrd;
                node.getSession().ISession.base64EncodedAuth = baseEncd;
                validProfile = 0;
            }
            else {
                return;
            }
            yield ussFileProvider.refreshElement(node);
            yield ussFileProvider.refresh();
        }
        else {
            validProfile = 0;
        }
        if (validProfile === 0) {
            const quickPickOptions = {
                placeHolder: `What would you like to create at ${node.fullPath}?`,
                ignoreFocusOut: true,
                canPickMany: false
            };
            const type = yield vscode.window.showQuickPick([extension.USS_DIR_CONTEXT, "File"], quickPickOptions);
            const isTopLevel = true;
            return createUSSNode(node, ussFileProvider, type, isTopLevel);
        }
    });
}
exports.createUSSNodeDialog = createUSSNodeDialog;
/**
 * Refreshes treeView
 *
 * @param {USSTree} ussFileProvider
 */
function refreshAllUSS(ussFileProvider) {
    return __awaiter(this, void 0, void 0, function* () {
        yield Profiles_1.Profiles.getInstance().refresh();
        ussFileProvider.mSessionNodes.forEach((sessNode) => {
            if (sessNode.contextValue === extension.USS_SESSION_CONTEXT) {
                utils.labelHack(sessNode);
                sessNode.children = [];
                sessNode.dirty = true;
                utils.refreshTree(sessNode);
            }
        });
        yield ussFileProvider.refresh();
    });
}
exports.refreshAllUSS = refreshAllUSS;
/**
 * Process for renaming a USS Node. This could be a Favorite Node
 *
 * @param {IZoweTreeNode} originalNode
 * @param {USSTree} ussFileProvider
 * @param {string} filePath
 */
function renameUSSNode(originalNode, ussFileProvider, filePath) {
    return __awaiter(this, void 0, void 0, function* () {
        // Could be a favorite or regular entry always deal with the regular entry
        const isFav = originalNode.contextValue.endsWith(extension.FAV_SUFFIX);
        const oldLabel = originalNode.label;
        const parentPath = originalNode.fullPath.substr(0, originalNode.fullPath.indexOf(oldLabel));
        // Check if an old favorite exists for this node
        const oldFavorite = isFav ? originalNode : ussFileProvider.mFavorites.find((temp) => (temp.shortLabel === oldLabel) && (temp.fullPath.substr(0, temp.fullPath.indexOf(oldLabel)) === parentPath));
        const newName = yield vscode.window.showInputBox({ value: oldLabel });
        if (newName && newName !== oldLabel) {
            try {
                let newNamePath = path.join(parentPath + newName);
                newNamePath = newNamePath.replace(/\\/g, "/"); // Added to cover Windows backslash issue
                yield ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getUssApi(originalNode.getProfile()).rename(originalNode.fullPath, newNamePath);
                yield deleteFromDisk(originalNode, filePath);
                originalNode.rename(newNamePath);
                if (oldFavorite) {
                    ussFileProvider.removeFavorite(oldFavorite);
                    oldFavorite.rename(newNamePath);
                    ussFileProvider.addFavorite(oldFavorite);
                }
            }
            catch (err) {
                utils.errorHandling(err, originalNode.mProfileName, localize(3, null) + err.message);
                throw (err);
            }
        }
    });
}
exports.renameUSSNode = renameUSSNode;
/**
 * Marks file as deleted from disk
 *
 * @param {ZoweUSSNode} node
 */
function deleteFromDisk(node, filePath) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            if (fs.existsSync(filePath)) {
                fs.unlinkSync(filePath);
            }
        }
        // tslint:disable-next-line: no-empty
        catch (err) {
        }
    });
}
exports.deleteFromDisk = deleteFromDisk;
function uploadDialog(node, ussFileProvider) {
    return __awaiter(this, void 0, void 0, function* () {
        const fileOpenOptions = {
            canSelectFiles: true,
            openLabel: "Upload Files",
            canSelectMany: true
        };
        const value = yield vscode.window.showOpenDialog(fileOpenOptions);
        yield Promise.all(value.map((item) => __awaiter(this, void 0, void 0, function* () {
            const isBinary = isbinaryfile_1.isBinaryFileSync(item.fsPath);
            if (isBinary) {
                yield uploadBinaryFile(node, item.fsPath);
            }
            else {
                const doc = yield vscode.workspace.openTextDocument(item);
                yield uploadFile(node, doc);
            }
        })));
        ussFileProvider.refresh();
    });
}
exports.uploadDialog = uploadDialog;
function uploadBinaryFile(node, filePath) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const localFileName = path.parse(filePath).base;
            const ussName = `${node.fullPath}/${localFileName}`;
            yield zowe.Upload.fileToUSSFile(node.getSession(), filePath, ussName, true);
        }
        catch (e) {
            utils.errorHandling(e, node.mProfileName, e.message);
        }
    });
}
exports.uploadBinaryFile = uploadBinaryFile;
function uploadFile(node, doc) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const localFileName = path.parse(doc.fileName).base;
            const ussName = `${node.fullPath}/${localFileName}`;
            yield ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getUssApi(node.getProfile()).putContents(doc.fileName, ussName);
        }
        catch (e) {
            utils.errorHandling(e, node.mProfileName, e.message);
        }
    });
}
exports.uploadFile = uploadFile;
/**
 * Copies full path for the selected Zowe USS node
 *
 * @param {ZoweUSSNode} node
 */
function copyPath(node) {
    return __awaiter(this, void 0, void 0, function* () {
        if (extension.ISTHEIA) {
            // Remove when Theia supports VS Code API for accessing system clipboard
            vscode.window.showInformationMessage(localize(4, null));
            return;
        }
        vscode.env.clipboard.writeText(node.fullPath);
    });
}
exports.copyPath = copyPath;

//# sourceMappingURL=ussNodeActions.js.map
