"use strict";
/*
* This program and the accompanying materials are made available under the terms of the *
* Eclipse Public License v2.0 which accompanies this distribution, and is available at *
* https://www.eclipse.org/legal/epl-v20.html                                      *
*                                                                                 *
* SPDX-License-Identifier: EPL-2.0                                                *
*                                                                                 *
* Copyright Contributors to the Zowe Project.                                     *
*                                                                                 *
*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode_1 = require("vscode");
const Profiles_1 = require("./Profiles");
const nls = require("vscode-nls");
const localize = nls.config({ messageFormat: nls.MessageFormat.file })(__filename);
function sortTreeItems(favorites, specificContext) {
    favorites.sort((a, b) => {
        if (a.contextValue === specificContext) {
            if (b.contextValue === specificContext) {
                return a.label.toUpperCase() > b.label.toUpperCase() ? 1 : -1;
            }
            else {
                return -1;
            }
        }
        else if (b.contextValue === specificContext) {
            return 1;
        }
        return a.label.toUpperCase() > b.label.toUpperCase() ? 1 : -1;
    });
}
exports.sortTreeItems = sortTreeItems;
/**
 * For no obvious reason a label change is often required to make a node repaint.
 * This function does this by adding or removing a blank.
 * @param {TreeItem} node - the node element
 */
function labelHack(node) {
    node.label = node.label.endsWith(" ") ? node.label.substring(0, node.label.length - 1) : node.label + " ";
}
exports.labelHack = labelHack;
function resolveQuickPickHelper(quickpick) {
    return __awaiter(this, void 0, void 0, function* () {
        return new Promise((c) => quickpick.onDidAccept(() => c(quickpick.activeItems[0])));
    });
}
exports.resolveQuickPickHelper = resolveQuickPickHelper;
// tslint:disable-next-line: max-classes-per-file
class FilterItem {
    constructor(text) {
        this.text = text;
    }
    get label() { return this.text; }
    get description() { return ""; }
    get alwaysShow() { return false; }
}
exports.FilterItem = FilterItem;
// tslint:disable-next-line: max-classes-per-file
class FilterDescriptor {
    constructor(text) {
        this.text = text;
    }
    get label() { return this.text; }
    get description() { return ""; }
    get alwaysShow() { return true; }
}
exports.FilterDescriptor = FilterDescriptor;
// tslint:disable-next-line: max-classes-per-file
class OwnerFilterDescriptor extends FilterDescriptor {
    constructor() {
        super("\uFF0B " + localize(0, null));
    }
}
exports.OwnerFilterDescriptor = OwnerFilterDescriptor;
// tslint:disable-next-line: max-classes-per-file
class JobIdFilterDescriptor extends FilterDescriptor {
    constructor() {
        super("\uFF0B " + localize(1, null));
    }
}
exports.JobIdFilterDescriptor = JobIdFilterDescriptor;
/*************************************************************************************************************
 * Returns array of all subnodes of given node
 *************************************************************************************************************/
function concatChildNodes(nodes) {
    let allNodes = new Array();
    for (const node of nodes) {
        allNodes = allNodes.concat(concatChildNodes(node.children));
        allNodes.push(node);
    }
    return allNodes;
}
exports.concatChildNodes = concatChildNodes;
/*************************************************************************************************************
 * Determine IDE name to display based on app environment
 *************************************************************************************************************/
function getAppName(isTheia) {
    return isTheia ? "Theia" : "VS Code";
}
exports.getAppName = getAppName;
/*************************************************************************************************************
 * Error Hanndling
 * @param {errorDetails} error.mDetails
 * @param {label} - additional information such as profile name, credentials, messageID etc
 * @param {moreInfo} - additional/customized error messages
 *************************************************************************************************************/
function errorHandling(errorDetails, label, moreInfo) {
    let httpErrCode = null;
    if (errorDetails.mDetails !== undefined) {
        httpErrCode = errorDetails.mDetails.errorCode;
    }
    switch (httpErrCode) {
        // tslint:disable-next-line: no-magic-numbers
        case 401: {
            vscode_1.window.showErrorMessage(localize(2, null) +
                localize(3, null) +
                `\n${label}\n` +
                localize(4, null));
            break;
        }
        default: {
            vscode_1.window.showErrorMessage(moreInfo + " " + errorDetails);
            break;
        }
    }
    return;
}
exports.errorHandling = errorHandling;
/*************************************************************************************************************
 * Refresh Profile and Session
 * @param {sessNode} IZoweTreeNode
 * @param {profile} IProfileLoaded
 *************************************************************************************************************/
function refreshTree(sessNode) {
    const allProf = Profiles_1.Profiles.getInstance().getProfiles();
    for (const profNode of allProf) {
        if (sessNode.getProfileName() === profNode.name) {
            sessNode.getProfile().profile = profNode.profile;
            const SessionProfile = profNode.profile;
            if (sessNode.getSession().ISession !== SessionProfile) {
                sessNode.getSession().ISession.user = SessionProfile.user;
                sessNode.getSession().ISession.password = SessionProfile.password;
                sessNode.getSession().ISession.base64EncodedAuth = SessionProfile.base64EncodedAuth;
                sessNode.getSession().ISession.hostname = SessionProfile.hostname;
                sessNode.getSession().ISession.port = SessionProfile.port;
                sessNode.getSession().ISession.rejectUnauthorized = SessionProfile.rejectUnauthorized;
            }
        }
    }
}
exports.refreshTree = refreshTree;

//# sourceMappingURL=utils.js.map
